@@ -53,6 +53,23 @@ def writetempfile(s):
     os.close(fd)
     return path
 
+# Since hg 5.8 (a3dced4b7b04), config dict is no longer a plain
+# {key: value} dict, but has metadata for each key.
+
+def metadict(src):
+    return {k: metavalue(v) for k, v in src.items()}
+
+def metavalue(v):
+    return (v, b'', 0)
+
+def plaindict(src):
+    return {k: plainvalue(v) for k, v in src.items()}
+
+def plainvalue(v):
+    value, source, level = v
+    assert isinstance(source, bytes)
+    assert isinstance(level, int)
+    return value
 
 def with_rconfig(f):
     f.__test__ = True
@@ -103,7 +120,7 @@ def check_contains():
 @with_both
 def check_getitem():
     c = newwconfig({b'foo.bar': b'x', b'foo.baz': b'y'})
-    assert dict(c[b'foo']) == {b'bar': b'x', b'baz': b'y'}
+    assert plaindict(c[b'foo']) == {b'bar': b'x', b'baz': b'y'}
     assert dict(c[b'unknown']) == {}
 
 @with_both
@@ -118,7 +135,7 @@ def check_set_followed_by_getitem_empty():
     c[b'unknown']
     c.set(b'unknown', b'foo', b'bar')
     assert c.get(b'unknown', b'foo') == b'bar'
-    assert c[b'unknown'][b'foo'] == b'bar'
+    assert plainvalue(c[b'unknown'][b'foo']) == b'bar'
 
 @with_both
 def check_dict_contains():
@@ -129,23 +146,23 @@ def check_dict_contains():
 @with_both
 def check_dict_getitem():
     c = newwconfig({b'foo.bar': b'x'})
-    assert c[b'foo'][b'bar'] == b'x'
+    assert plainvalue(c[b'foo'][b'bar']) == b'x'
     with pytest.raises(KeyError):
         c[b'foo'][b'baz']
 
 @with_both
 def check_dict_setitem():
     c = newwconfig({b'foo.bar': b'x'})
-    c[b'foo'][b'bar'] = b'y'
-    c[b'foo'][b'baz'] = b'z'
-    assert c[b'foo'][b'bar'] == b'y'
-    assert c[b'foo'][b'baz'] == b'z'
+    c[b'foo'][b'bar'] = metavalue(b'y')
+    c[b'foo'][b'baz'] = metavalue(b'z')
+    assert c[b'foo'][b'bar'] == metavalue(b'y')
+    assert c[b'foo'][b'baz'] == metavalue(b'z')
 
 @with_wconfig  # original config doesn't preserve the order
 def check_dict_setitem_preserve_order():
     c = newwconfig([(b'foo.bar', b'x'), (b'foo.baz', b'y')])
     assert list(c[b'foo']) == [b'bar', b'baz']
-    c[b'foo'][b'bar'] = b'z'
+    c[b'foo'][b'bar'] = metavalue(b'z')
     assert list(c[b'foo']) == [b'bar', b'baz']
 
 @with_both
@@ -162,8 +179,8 @@ def check_dict_len():
 def check_dict_update():
     c = newwconfig({b'foo.bar': b'x', b'foo.baz': b'y'})
     c[b'foo'].update(newwconfig({b'foo.bar': b'z', b'foo.baz': b'w'})[b'foo'])
-    assert c[b'foo'][b'bar'] == b'z'
-    assert c[b'foo'][b'baz'] == b'w'
+    assert plainvalue(c[b'foo'][b'bar']) == b'z'
+    assert plainvalue(c[b'foo'][b'baz']) == b'w'
 
 @with_both
 def check_dict_delitem():
@@ -271,13 +288,13 @@ def check_read_write_parsing_error():
 @with_wconfig
 def check_write_after_dict_setitem():
     c = newwconfig({b'foo.bar': b'x'})
-    c[b'foo'][b'bar'] = b'y'
+    c[b'foo'][b'bar'] = metavalue(b'y')
     assert written(c).rstrip() == b'[foo]\nbar = y'
 
 @with_wconfig
 def check_write_after_dict_update():
     c = newwconfig({b'foo.bar': b'x'})
-    c[b'foo'].update({b'bar': b'y'})
+    c[b'foo'].update(metadict({b'bar': b'y'}))
     assert written(c).rstrip() == b'[foo]\nbar = y'
 
 @with_wconfig
