@@ -2,7 +2,7 @@
 
 from __future__ import absolute_import
 
-from nose.plugins.skip import SkipTest
+import unittest
 
 from tortoisehg.hgqt.qtgui import (
     QApplication,
@@ -10,4 +10,4 @@ from tortoisehg.hgqt.qtgui import (
 
 def setup_module():
     if not isinstance(QApplication.instance(), QApplication):
-        raise SkipTest
+        raise unittest.SkipTest
