@@ -30,6 +30,7 @@ from mercurial import (
 from mercurial.utils import (
     procutil,
     stringutil,
+    urlutil,
 )
 
 from ..util import (
@@ -51,7 +52,7 @@ from . import (
 )
 
 try:
-    from tortoisehg.util.config import nofork as config_nofork
+    from tortoisehg.util.config import nofork as config_nofork  # pytype: disable=import-error
 except ImportError:
     config_nofork = None
 
@@ -390,7 +391,7 @@ def runcommand(ui, args):
             else:
                 path, bundle = s
             cmdoptions[b'bundle'] = os.path.abspath(bundle)
-        path = ui.expandpath(path)
+        path = urlutil.get_clone_path(ui, path)[0]
         # TODO: replace by abspath() if chdir() isn't necessary
         try:
             os.chdir(path)
@@ -454,10 +455,10 @@ def _runcommand(ui, options, cmd, cmdfunc):
         output = ui.config(b'profiling', b'output')
 
         if output:
-            path = ui.expandpath(output)
+            path = util.expandpath(output)
             ostream = open(path, 'wb')
         else:
-            ostream = sys.stderr
+            ostream = procutil.stderr
 
         try:
             from mercurial import lsprof
@@ -600,7 +601,7 @@ def backout(ui, repoagent, *pats, **opts):
     elif len(pats) == 1:
         rev = pats[0]
     else:
-        rev = 'tip'
+        rev = b'tip'
     repo = repoagent.rawRepo()
     rev = scmutil.revsingle(repo, rev).rev()
     msg = backoutmod.checkrev(repo, rev)
@@ -770,7 +771,8 @@ def graft(ui, repoagent, *revs, **opts):
     repo = repoagent.rawRepo()
     revs = list(revs)
     revs.extend(opts['rev'])
-    if not os.path.exists(repo.vfs.join(b'graftstate')) and not revs:
+
+    if not hglib.graft_in_progress(repo) and not revs:
         raise error.Abort(_('You must provide revisions to graft'))
     return graftmod.GraftDialog(repoagent, None,
         source=[hglib.tounicode(rev) for rev in revs])
@@ -1107,13 +1109,13 @@ def postreview(ui, repoagent, *pats, **opts):
     from tortoisehg.hgqt import postreview as postreviewmod
     repo = repoagent.rawRepo()
     if b'reviewboard' not in repo.extensions():
-        url = 'https://www.mercurial-scm.org/wiki/ReviewboardExtension'
+        url = b'https://www.mercurial-scm.org/wiki/ReviewboardExtension'
         raise error.Abort(_('reviewboard extension not enabled'),
                           hint=(_('see <a href="%(url)s">%(url)s</a>')
-                                % {'url': url}))
+                                % {b'url': url}))
     revs = opts.get('rev') or None
     if not revs and len(pats):
-        revs = pats[0]
+        revs = pats
     if not revs:
         raise error.Abort(_('no revisions specified'))
     return postreviewmod.PostReviewDialog(repo.ui, repoagent, revs)
@@ -1152,13 +1154,32 @@ def rebase(ui, repoagent, *pats, **opts):
     """rebase dialog"""
     from tortoisehg.hgqt import rebase as rebasemod
     repo = repoagent.rawRepo()
-    if os.path.exists(repo.vfs.join(b'rebasestate')):
+    if hglib.rebase_in_progress(repo):
         # TODO: move info dialog into RebaseDialog if possible
         qtlib.InfoMsgBox(hglib.tounicode(_('Rebase already in progress')),
                          hglib.tounicode(_('Resuming rebase already in '
                                            'progress')))
     elif not opts['source'] or not opts['dest']:
         raise error.Abort(_('You must provide source and dest arguments'))
+
+    for name in ('source', 'dest'):
+        revspec = opts[name]
+        if revspec:
+            revs = scmutil.revrange(repo, [revspec])
+
+            if not revs:
+                raise error.Abort(
+                    _("empty revision range in --%s") % hglib.fromunicode(name)
+                )
+
+            if len(revs) > 1:
+                raise error.Abort(
+                    _("multiple revisions cannot be given to --%s")
+                    % hglib.fromunicode(name)
+                )
+
+            opts[name] = revs.first()
+
     return rebasemod.RebaseDialog(repoagent, None, **opts)
 
 @command(b'rejects', [], _('thg rejects [FILE]'))
@@ -1296,7 +1317,8 @@ def strip(ui, repoagent, *pats, **opts):
         rev = opts.get('rev')
     elif len(pats) == 1:
         rev = pats[0]
-    return thgstrip.createStripDialog(repoagent, rev=rev, opts=opts)
+    return thgstrip.createStripDialog(repoagent, rev=hglib.tounicode(rev),
+                                      opts=opts)
 
 @command(b'sync|synchronize',
     [(b'B', b'bookmarks', False, _('open the bookmark sync window'))],
@@ -1403,6 +1425,15 @@ def vdiff(ui, repoagent, *pats, **opts):
     if opts.get('bundle'):
         repo = thgrepo.repository(ui, opts.get('bundle'))
     pats = hglib.canonpaths(pats)
+
+    rev = opts.get('rev')
+    if rev:
+        opts['rev'] = [hglib.tounicode(r) for r in rev]
+
+    rev = opts.get('change')
+    if rev:
+        opts['change'] = hglib.tounicode(rev)
+
     return visdiff.visualdiff(ui, repo, pats, opts)
 
 @command(b'version',
