@@ -823,6 +823,10 @@ INFO = (
           '<b>Note</b>: This layouter colors edges using branch information '
           'and does not display graft edges, regardless of whether they are '
           'requested or not.')),
+    _fi(_('Copy Hash to Selection'), 'tortoisehg.copy_hash_selection',
+        genBoolRBGroup,
+        _('When a revision is selected in Workbench, copy hash to primary '
+          'selection clipboard (X11). Default: False')),
     )),
 ({'name': 'commit', 'label': _('Commit', 'config item'), 'icon': 'hg-commit'},
  (
@@ -1858,7 +1862,7 @@ class SettingsForm(QWidget):
                 return None
             sect, key = pycompat.sysbytes(cpath).split(b'.', 1)
             try:
-                return self.ini[sect][key]
+                return self.ini.get(sect, key)
             except KeyError:
                 pass
 
