@@ -19,6 +19,7 @@ from hgext import mq as mqmod
 from mercurial import (
     cmdutil,
     config,
+    context,
     dispatch as dispatchmod,
     encoding,
     error,
@@ -77,6 +78,8 @@ if TYPE_CHECKING:
         localrepo,
     )
     from .typelib import (
+        DiffTools,
+        HgContext,
         IniConfig,
     )
 
@@ -247,6 +250,12 @@ def _firstchangectx(repo):
         return repo[rev]
     return repo[nullrev]
 
+def revsymbol(repo, rev):
+    # type: (localrepo.localrepository, bytes) -> HgContext
+    ctx = scmutil.revsymbol(repo, rev)
+    assert isinstance(ctx, (context.changectx, context.workingctx))
+    return ctx
+
 def shortrepoid(repo):
     # type: (...) -> str
     """Short hash of the first root changeset; can be used for settings key"""
@@ -287,7 +296,7 @@ def getrevisionlabel(repo, rev):
 
     ctx = repo[rev]
     label = _getfirstrevisionlabel(repo, ctx)
-    if label and ctx == scmutil.revsymbol(repo, label):
+    if label and ctx == revsymbol(repo, label):
         return tounicode(label)
 
     return '%d' % rev
@@ -318,7 +327,7 @@ def gitcommit_full(ctx):
     returns the complete git commit hash of the current revision
     """
 
-    repo = ctx._repo
+    repo = ctx.repo()
 
     if b'hggit' not in repo.extensions():
         return None
@@ -365,6 +374,28 @@ def readgraftstate(repo):
 mergestate = mergestatemod.mergestate
 readmergestate = mergestate.read
 
+
+def operation_in_progress(repo, statename):
+    # type: (localrepo.localrepository, bytes) -> bool
+    """Indicates if a long term operation is in progress"""
+    return statemod.cmdstate(repo, statename).exists()
+
+
+def graft_in_progress(repo):
+    # type: (localrepo.localrepository) -> bool
+    return operation_in_progress(repo, b"graftstate")
+
+
+def pick_in_progress(repo):
+    # type: (localrepo.localrepository) -> bool
+    return operation_in_progress(repo, b"pickstate")
+
+
+def rebase_in_progress(repo):
+    # type: (localrepo.localrepository) -> bool
+    return operation_in_progress(repo, b"rebasestate")
+
+
 def readundodesc(repo):
     # type: (...) -> Tuple[Text, int]
     """Read short description and changelog size of last transaction"""
@@ -546,9 +577,9 @@ def mergetools(ui):
     return values
 
 
-_difftools = None
+_difftools = None  # type: Optional[DiffTools]
 def difftools(ui):
-    # TODO: type annotation
+    # type: (uimod.ui) -> DiffTools
     '''Return mapping from tool name to tuples with tool name,
     diff args, and merge args, all as bytecode strings'''
     global _difftools
@@ -566,7 +597,7 @@ def difftools(ui):
             mergeopts = []
         return diffopts, mergeopts
 
-    tools = {}
+    tools = {}  # type: DiffTools
     for cmd, path in ui.configitems(b'extdiff'):
         if cmd.startswith(b'cmd.'):
             cmd = cmd[4:]
@@ -575,7 +606,7 @@ def difftools(ui):
             diffopts = ui.config(b'extdiff', b'opts.' + cmd)
             diffopts = pycompat.shlexsplit(diffopts)
             diffopts, mergeopts = fixup_extdiff(diffopts)
-            tools[cmd] = [path, diffopts, mergeopts]
+            tools[cmd] = (path, diffopts, mergeopts)
         elif cmd.startswith(b'opts.'):
             continue
         else:
@@ -586,7 +617,7 @@ def difftools(ui):
             else:
                 path, diffopts = cmd, []
             diffopts, mergeopts = fixup_extdiff(diffopts)
-            tools[cmd] = [path, diffopts, mergeopts]
+            tools[cmd] = (path, diffopts, mergeopts)
     for t in mergetools(ui):
         if t.startswith(b'internal:'):
             continue
@@ -825,11 +856,11 @@ def extractchoices(prompttext):
     return msg, pycompat.ziplist(resps, choices)
 
 def displaytime(date):
-    # type: (Optional[Tuple[int, int]]) -> bytes
+    # type: (Optional[Tuple[float, int]]) -> bytes
     return dateutil.datestr(date, b'%Y-%m-%d %H:%M:%S %1%2')
 
 def utctime(date):
-    # type: (Tuple[int, int]) -> str
+    # type: (Tuple[float, int]) -> str
     return time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(date[0]))
 
 agescales = [
@@ -905,13 +936,20 @@ def user(ctx):
 def getestimatedsize(fctx):
     # type: (...) -> int
     """Return the size of the given fctx without loading the revision text"""
+    FILE_SIZE_UNKNOWN = 2147483647
+
     if fctx.rev() is None:
         return fctx.size()
     else:
         # fctx.size() can read all data into memory in rename cases so
         # we read the size directly from the filelog, this is deeper
         # under the API than I prefer to go, but seems necessary
-        return fctx._filelog._revlog.rawsize(fctx.filerev())
+        try:
+            return fctx._filelog._revlog.rawsize(fctx.filerev())
+        except AttributeError:
+            # _filelog is not available when using the remotefilelog extension
+            # on a shallow repo
+            return FILE_SIZE_UNKNOWN
 
 def get_revision_desc(fctx, curpath=None):
     # type: (Any, Optional[bytes]) -> pycompat.unicode
@@ -971,7 +1009,7 @@ def getDeepestSubrepoContainingFile(wfile, ctx):
             stype = ctx.substate[wsub][2]  # type: bytes
             if stype != b'hg':
                 continue
-            if not os.path.exists(ctx._repo.wjoin(wsub)):
+            if not os.path.exists(ctx.repo().wjoin(wsub)):
                 # Maybe the repository does not exist in the working copy?
                 continue
             try:
@@ -1250,6 +1288,8 @@ def buildcmdargs(name, *args, **opts):
     ['qpush']
     >>> buildcmdargs('update', '')
     ['update', '']
+    >>> buildcmdargs('update', 25)
+    ['update', '25']
 
     type conversion to string:
 
